@@ -6,19 +6,22 @@ as.ppp.SpatialPoints = function(X) {
         colnames(bb) <- NULL
 	W = owin(bb[1,], bb[2,])
 	cc = coordinates(X)
-	return(ppp(cc[,1], cc[,2], window = W, marks = NULL))
+	return(ppp(cc[,1], cc[,2], window = W, marks = NULL, check=FALSE))
 }
 
 setAs("SpatialPoints", "ppp", function(from) as.ppp.SpatialPoints(from))
 
 as.ppp.SpatialPointsDataFrame = function(X) {
 	require(spatstat)
-	W = owin(bbox(X)[1,], bbox(X)[2,])
-	if (ncol(X) > 1)
-	  stop("ppp objects only accept a single attribute column; please select one")
-	marks = X[[1]]
-	cc = coordinates(X)
-	return(ppp(cc[,1], cc[,2], window = W, marks = marks))
+	bb <- bbox(X)
+        colnames(bb) <- NULL
+	W  <- owin(bb[1,], bb[2,])
+	nc <- ncol(X)
+        marks <- if(nc == 0) NULL else X[[1]]
+        if(nc > 1)
+          warning(paste(nc-1, "columns of data frame discarded"))
+	cc <- coordinates(X)
+	return(ppp(cc[,1], cc[,2], window = W, marks = marks, check=FALSE))
 }
 
 setAs("SpatialPointsDataFrame", "ppp", function(from) as.ppp.SpatialPointsDataFrame(from))
@@ -56,12 +59,13 @@ setAs("SpatialPolygons", "owin", function(from) as.owin.SpatialPolygons(from))
 
 as.psp.Line <- function(from, ..., window=NULL, marks=NULL, fatal) {
   xy <- from@coords
-  xrange <- range(xy[,1])
-  yrange <- range(xy[,2])
   df <- as.data.frame(cbind(xy[-nrow(xy), , drop=FALSE], xy[-1, ,
 drop=FALSE]))
-  if(is.null(window))
+  if(is.null(window)) {
+    xrange <- range(xy[,1])
+    yrange <- range(xy[,2])
     window <- owin(xrange, yrange)
+  }
   return(as.psp(df, window=window, marks=marks))
 }
 
@@ -78,16 +82,16 @@ as.psp.Lines <- function(from, ..., window=NULL, marks=NULL, fatal) {
 setAs("Lines", "psp", function(from) as.psp.Lines(from))
 
 as.psp.SpatialLines <- function(from, ..., window=NULL, marks=NULL, fatal) {
-  lin <- from@lines
-  y <- lapply(lin, as.psp.Lines)
-  id <- unlist(lapply(lin, function(s) { s@ID }))
-  if(is.null(marks))
-    for(i in seq(y)) 
-      marks(y[[i]]) <- id[i]
   if(is.null(window)) {
     w <- from@bbox
     window <- owin(w[1,], w[2,])
   }
+  lin <- from@lines
+  y <- lapply(lin, as.psp.Lines, window=window)
+  id <- unlist(lapply(lin, function(s) { s@ID }))
+  if(is.null(marks))
+    for(i in seq(y)) 
+      marks(y[[i]]) <- id[i]
   z <- do.call("superimposePSP", list(y, window=window))
   if(!is.null(marks))
     marks(z) <- marks
@@ -102,13 +106,13 @@ as.psp.SpatialLinesDataFrame <- function(from, ..., window=NULL, marks=NULL, fat
   if(is.null(marks)) {
     # extract marks from first column of data frame
     df <- from@data
-    if(is.null(df))
+    if(is.null(df) || (nc <- ncol(df)) == 0)
       return(z)
-    if((nc <- ncol(df)) > 1) 
+    if(nc > 1) 
       warning(paste(nc-1, "columns of data frame discarded"))
     marx <- df[,1]
     nseg.Line  <- function(x) { return(nrow(x@coords)-1) }
-    nseg.Lines <- function(x) { return(unlist(lapply(x@Lines, nseg.Line))) }
+    nseg.Lines <- function(x) { return(sum(unlist(lapply(x@Lines, nseg.Line)))) }
     nrep <- unlist(lapply(y@lines, nseg.Lines))
     marks(z) <- rep(marx, nrep)
   }
